(function ($, undefined) {
  $.fn.tableSort = function (options) {

    this.each(function(options) {
      var self = $(this);
      var settings = $.extend({}, $.fn.tableSort.defaults, options);

      var naturalSort = require('./naturalSort.js');

      self.find('tbody tr').each(function(i) {
        $(this).attr('data-original-sort-index', i).data('original-sort-index', i);
      });

      self.find('th[data-sortable=true]').each(function() {
        var $this = $(this);

        var $sort = $('<a></a>').attr({
          class: 'tableSortSortable',
          href: '#',
          'data-column-index': this.cellIndex
        })
        .data('column-index', this.cellIndex)
        .html('<i class="'+ settings.sortableIconClass +'"></i>');

        $this.append(' ').append($sort);
      });

      self.on('click', '.tableSortSortable', function(e) {
        e.preventDefault();
        
        var $this = $(this);
        switch ($this.data('sort-direction')) {
          case 'asc':
            $this
              .attr('sort-direction', 'desc')
              .data('sort-direction', 'desc');
              sortColumnDesc($this.data('column-index'));
            break;
          case 'desc':
            $this
              .attr('sort-direction', 'sortable')
              .data('sort-direction', 'sortable');
              clearSort();
            break;
          default:
            $this
              .attr('sort-direction', 'asc')
              .data('sort-direction', 'asc');
              sortColumnAsc($this.data('column-index'));
            break;
        }

        self.find('.fa-sort-asc, .fa-sort-desc').attr('class', settings.sortableIconClass);
        $this.find('i').attr('class', settings[$this.data('sort-direction') + 'IconClass']);
      });

      function sortColumnAsc(columnIndex) {
        var $rows = self.find('tbody tr');

        var rows = [];

        $rows.each(function() {
          var sortValue = $($(this).find('td')[columnIndex]).data('sort-value');
          rows.push({
            key: sortValue,
            value: this
          });
        });

        rows.sort(function (a, b) {
          return naturalSort(a.key, b.key);
        });

        var $tbody = self.find('tbody');
        $tbody.empty();

        rows.forEach(function (row) {
          $tbody.append(row.value);
        });
      }

      function sortColumnDesc(columnIndex) {
        var $rows = self.find('tbody tr');

        var rows = [];

        $rows.each(function() {
          var sortValue = $($(this).find('td')[columnIndex]).data('sort-value');
          rows.push({
            key: sortValue,
            value: this
          });
        });

        rows.sort(function (a, b) {
          return naturalSort(b.key, a.key);
        });

        var $tbody = self.find('tbody');
        $tbody.empty();

        rows.forEach(function (row) {
          $tbody.append(row.value);
        });
      }

      function clearSort() {
        var $rows = self.find('tbody tr');

        var rows = [];

        $rows.each(function() {
          var sortValue = $(this).data('original-sort-index');
          rows.push({
            key: sortValue,
            value: this
          });
        });

        rows.sort(function (a, b) {
          return naturalSort(a.key, b.key);
        });

        var $tbody = self.find('tbody');
        $tbody.empty();

        rows.forEach(function (row) {
          $tbody.append(row.value);
        });
      }
    });
  };

  $.fn.tableSort.defaults = {
    sortDirection: 'asc',
    sortableIconClass: 'fa fa-sort',
    ascIconClass: 'fa fa-sort-asc',
    descIconClass: 'fa fa-sort-desc',
  };
})(jQuery);
